# ST-Sim Self-directed course
# Exercise 10 - Running ST-Sim from R

# Last edited by: ApexRMS, February 20, 2020

#-----------------------------------------------------------------------------------
# IMPORTANT DOWNLOAD INSTRUCTIONS:
#
# Be sure to use all the latest versions of software for this Exercise:
#   1. Follow the instructions at www.syncrosim.com/download to first download and install:
#       -The latest version of SyncroSim (version 2.2.5 or higher)
#       -The latest version of rsyncrosim (version 1.1.9 or higher)
#   2. Using the "File | Packages" menu in SyncroSim for Windows, click the "Updates" button to ensure 
#      you have the latest version of the "stsim" package (version 3.2.13 or higher)
#-----------------------------------------------------------------------------------

#-----------------------------------------------------------------------------------
# Exercise 10 Task 1: Post-processing SyncroSim output
#-----------------------------------------------------------------------------------

# In this Task you will learn how to post-process output from a SyncroSim model run

# Start by opening the following library in SyncroSim for Windows:
# Folder: Documents/SyncroSim/Course/Exercise 10/
# Filename: Exercise 10 Demo Library.ssim     (*** note the word "Demo" in the file name)
#
# Look at the Project and Scenarios in this Library
# They are the same scenarios you have been working with all course
#
# Now run the code below, one line at a time  (move to line and hit Ctrl-Enter to run it in RStudio)
#
#-----------------------------------------------------------------------------------
# Load required packages - if you get an error then you must install these packages first

library(rsyncrosim)  # package for working with SyncroSim - must be version 1.1.7 or higher
library(raster)      # raster package required to work with spatial data

#-----------------------------------------------------------------------------------
# Set the current working directory

# Check the current working directory. If this is not correct, use the "Session | Set Working Directory" menu in RStudio to set it
getwd()

# Lists the files in the current working directory - make sure you see the file "Exercise 10 Demo Library.ssim"
list.files()

#-----------------------------------------------------------------------------------
# Open an existing SyncroSim Library

# Note that if the project does not exist already, then this command will create a new library
myLibrary = ssimLibrary("Exercise 10 Demo Library.ssim")

#-----------------------------------------------------------------------------------
# List the Projects in this Library, then load a project

# If this command returns no projects, SyncroSim created a new blank library - change your working folder above and try again
project(myLibrary)
myProject = project(myLibrary, project="Exercise 10 - Demo")
myProject

#-----------------------------------------------------------------------------------
# List the Scenarios and Datasheets in this Project

# Note that this list includes Result Scenarios
scenario(myProject)

# Load the Result Scenario containing the output we wish to post-process - in this case for the "no harvest" scenario
myResultScenario = scenario(myProject, scenario=29)
myResultScenario

# List the names of all the available Datasheets for "stsim" SyncroSim Libraries (including both inputs and outputs)
myDataSheetGuide = datasheet(myResultScenario, summary = TRUE)  # The list is long! 

head(myDataSheetGuide)  # You can view the full list of datasheets by clicking on "datasheetGuide" in top right "Environment" pane of RStudio

#-----------------------------------------------------------------------------------
# Get the tabular state output and plot it

# Get the raw tabular state output from its datasheet and load it into R as a dataframe (see myDatasheetGuide for valid names)
outputRaw = datasheet(myResultScenario, name="stsim_OutputStratumState")

# Show a bit of this dataframe - it is raw output, so it has over 200,000 rows - however you can view it also using RStudio "Environment" pane
tail(outputRaw)

# Now we can post-process this output however we like in R
# We use only base R functions to summarize and plot here - other packages can help with this (e.g. ddplyr, ggplot2, rasterVis)

# First we sum the Amount field over all fields except Iteration, Timestep and StateLabelXID
outputSum = aggregate(outputRaw["Amount"], by=outputRaw[c("Iteration", "Timestep", "StateLabelXID")], sum)
tail(outputSum)

# Then we average over Iterations
outputMean = aggregate(outputSum["Amount"], by=outputSum[c("Timestep", "StateLabelXID")], mean)
tail(outputMean)

# And finally plot the mean conifer area over time
conifer = subset(outputMean, StateLabelXID=="Conifer")
x=conifer$Timestep
y=conifer$Amount
plot(x,y)

#-----------------------------------------------------------------------------------
# Get the spatial state output and plot it

# Get the raw raster state output from its datasheet and load it into R as a dataframe (see name in myDatasheetGuide)
# Get only the raster data for iteration 1 and timestep 20
outputRaster = datasheetRaster(myResultScenario, datasheet="stsim_OutputSpatialState", iteration=1, timestep=20)
outputRaster

plot(outputRaster)        # Plot the raster
freq(outputRaster)        # Count for each State Class ID value on the raster
datasheet(myResultScenario, name="stsim_StateClass")   # State Classes and their ID values (from the Project Properties)


#-----------------------------------------------------------------------------------
# Exercise 10 Task 2: Modifying and running a SyncroSim model
#-----------------------------------------------------------------------------------

# In this Task you will learn how to modify and run a SyncroSim model

#-----------------------------------------------------------------------------------
# Make a copy of an existing Scenario

# List the Scenarios in this Project
scenario(myProject)

# Get the Scenario with no harvest
myScenario = scenario(myProject, scenario = 29)
name(myScenario)

# Make a copy of this scenario and give the copy a new name
myScenario2 = scenario(myProject, scenario="Harvest 2000 ac/yr", sourceScenario=myScenario)
scenario(myProject)

#-----------------------------------------------------------------------------------
# Edit the inputs for this new Scenario

# Create an empty template dataframe corresponding to a Transition Target Datasheet for this new scenario
mySheetData = datasheet(myScenario2, name="stsim_TransitionTarget", empty=TRUE)
mySheetData

# Now add a new row to this dataframe specifying 2000 ac/hr Harvest
mySheetData = addRow(mySheetData, data.frame(TransitionGroupID="Harvest",Amount=2000))
mySheetData

# Save this dataframe back to the new Scenario's datasheet
saveDatasheet(myScenario2, data=mySheetData, name="stsim_TransitionTarget")

# Go to SyncroSim for Windows and select "File | Refresh All Libraries"
# While still in SyncroSim for Windows, double-click on the scenario "Harvest 2000 ac/yr", then click on the Transition Targets tab
# to see the new target for this scenario

#-----------------------------------------------------------------------------------
# Run this new Scenario and display new Results Scenario

resultSummary = run(myProject, scenario="Harvest 2000 ac/yr", jobs=6, summary=T)   # Uses multiprocessing
resultSummary

# Return to SyncroSim for Windows and select "File | Refresh All Libraries" again to see that Results have been created for this scenario
# While still in SyncroSim for Windows, review the model inputs and outputs (by selecting "Add Results" for this scenario,
# then view Charts and Maps)


#-----------------------------------------------------------------------------------
# Exercise 10 Task 3: Creating a new SyncroSim model
#-----------------------------------------------------------------------------------

# In this Task you will learn how to create a new SyncroSim model from scratch

#-----------------------------------------------------------------------------------
# Display internal names of all the library's datasheets - corresponds to the the 'File-Library Properties' menu in SyncroSim
datasheet(myLibrary, summary=T)

# Get the current values for the Library's Backup Datasheet
sheetData = datasheet(myLibrary, name="core_Backup")                        # Get the current backup settings for the library
sheetData

# Modify the values for the Library's Backup Datasheet
sheetData$IncludeOutput=TRUE  # Add output to the backup
saveDatasheet(myLibrary, data=sheetData, name="core_Backup")                       # Save the new dataframe back to the library
datasheet(myLibrary, "core_Backup")

#-----------------------------------------------------------------------------------
# Create new Project and setup the Project Properties

myProject = project(myLibrary, project="Exercise 10 - New")
project(myLibrary, summary=TRUE)     # Lists the projects in this library

# Display internal names of all the project's datasheets - corresponds to the Project Properties in SyncroSim
datasheet(myProject, summary=T)

#-----------------------------------------------------------------------------------
# Edit the Project Properties - corresponds to the 'Project-Properties' menu in SyncroSim

# Terminology
sheetData = datasheet(myProject, "stsim_Terminology")
sheetData
sheetData$AmountUnits[1] = "Hectares"
sheetData$StateLabelX[1] = "Forest Type"
saveDatasheet(myProject, sheetData, "stsim_Terminology")
datasheet(myProject, "stsim_Terminology")

# Stratum
sheetData = datasheet(myProject, "stsim_Stratum", empty=T)   # Returns empty dataframe with only required column(s)
sheetData = addRow(sheetData, "Entire Forest")
saveDatasheet(myProject, sheetData, "stsim_Stratum", force=T)
datasheet(myProject, "stsim_Stratum", optional=T)   # Returns entire dataframe, including optional columns

# First State Class Label (i.e. Forest Types)
forestTypes = c("Coniferous", "Deciduous", "Mixed")
saveDatasheet(myProject, data.frame(Name=forestTypes), "stsim_StateLabelX", force=T)

# Second State Label (not used)
saveDatasheet(myProject, data.frame(Name="All"), "stsim_StateLabelY", force=T)

# Transition Types
transitionTypes = data.frame(Name=c("Fire", "Harvest", "Succession"), ID=c(1,2,3))
saveDatasheet(myProject, transitionTypes, "stsim_TransitionType", force=T)

# State Classes
stateClasses = datasheet(myProject, name="stsim_StateClass")
stateClasses = addRow(stateClasses, data.frame(Name="Coniferous:All", StateLabelXID="Coniferous", StateLabelYID="All"))
stateClasses = addRow(stateClasses, data.frame(Name="Mixed:All", StateLabelXID="Mixed", StateLabelYID="All"))
stateClasses = addRow(stateClasses, data.frame(Name="Deciduous:All", StateLabelXID="Deciduous", StateLabelYID="All"))
saveDatasheet(myProject, stateClasses, "stsim_StateClass", force=T)

# Ages
ageFrequency = 1
ageMax = 101
ageGroups = c(20,40,60,80,100)
saveDatasheet(myProject, data.frame(Frequency=ageFrequency, MaximumAge=ageMax), "stsim_AgeType", force=T)
saveDatasheet(myProject, data.frame(MaximumAge=ageGroups), "stsim_AgeGroup", force=T)

# Go to SyncroSim for Windows and Refresh; check that the Library and Project Properties (for the new "Exercise 10 - New" Project) are correct

#-----------------------------------------------------------------------------------
# Create new Scenario and setup the Scenario Properties

# Create a new SyncroSim "No Harvest" scenario
myScenario = scenario(myProject, "No Harvest")

# Display the internal names of all the scenario datasheets
myDataSheetGuide = subset(datasheet(myScenario, summary=T), scope=="scenario")   # Generate list of all Datasheets as reference

# Edit the scenario datasheets:

# Run Control - Note that we will set this as a non-spatial run
sheetName = "stsim_RunControl"
sheetData = data.frame(MaximumIteration=6, MinimumTimestep=0, MaximumTimestep=20, isSpatial=F)
saveDatasheet(myScenario, sheetData, sheetName)

# States
sheetName = "stsim_DeterministicTransition"
sheetData = datasheet(myScenario, sheetName , optional=T, empty=T)
sheetData = addRow(sheetData, data.frame(StateClassIDSource="Coniferous:All",StateClassIDDest="Coniferous:All",AgeMin=21,Location="C1"))
sheetData = addRow(sheetData, data.frame(StateClassIDSource="Deciduous:All",StateClassIDDest="Deciduous:All",Location="A1"))
sheetData = addRow(sheetData, data.frame(StateClassIDSource="Mixed:All",StateClassIDDest="Mixed:All",AgeMin=11,Location="B1"))
saveDatasheet(myScenario, sheetData, sheetName)

# Probabilistic Transitions
sheetName = "stsim_Transition"
sheetData = datasheet(myScenario, sheetName , optional=T, empty=T)
sheetData = addRow(sheetData, data.frame(StateClassIDSource="Coniferous:All",StateClassIDDest="Deciduous:All", TransitionTypeID="Fire",Probability=0.01))
sheetData = addRow(sheetData, data.frame(StateClassIDSource="Coniferous:All",StateClassIDDest="Deciduous:All", TransitionTypeID="Harvest",Probability=1,AgeMin=40))
sheetData = addRow(sheetData, data.frame(StateClassIDSource="Deciduous:All",StateClassIDDest="Deciduous:All", TransitionTypeID="Fire",Probability=0.002))
sheetData = addRow(sheetData, data.frame(StateClassIDSource="Deciduous:All",StateClassIDDest="Mixed:All", TransitionTypeID="Succession",Probability=0.1,AgeMin=10))
sheetData = addRow(sheetData, data.frame(StateClassIDSource="Mixed:All",StateClassIDDest="Deciduous:All", TransitionTypeID="Fire",Probability=0.005))
sheetData = addRow(sheetData, data.frame(StateClassIDSource="Mixed:All",StateClassIDDest="Coniferous:All", TransitionTypeID="Succession",Probability=0.1,AgeMin=20))
saveDatasheet(myScenario, sheetData, sheetName)

# Initial Conditions: Non-spatial

sheetName = "stsim_InitialConditionsNonSpatial"
sheetData = data.frame(TotalAmount=100, NumCells=100, CalcFromDist=F)
saveDatasheet(myScenario, sheetData, sheetName)
datasheet(myScenario, sheetName)

sheetName = "stsim_InitialConditionsNonSpatialDistribution"
sheetData = data.frame(StratumID="Entire Forest", StateClassID="Coniferous:All", RelativeAmount=1)
saveDatasheet(myScenario, sheetData, sheetName)
datasheet(myScenario, sheetName)

# Transition targets - set harvest to 0 for this scenario
saveDatasheet(myScenario, data.frame(TransitionGroupID="Harvest [Type]", Amount=0), "stsim_TransitionTarget")

# Output options - non-spatial only
datasheet(myScenario, "stsim_OutputOptions")
sheetData = data.frame(SummaryOutputSC=T, SummaryOutputSCTimesteps=1,
                       SummaryOutputTR=T, SummaryOutputTRTimesteps=1)
saveDatasheet(myScenario, sheetData, "stsim_OutputOptions")

# Go to SyncroSim for Windows and Refresh; check the Scenario's Properties are all correct

#-----------------------------------------------------------------------------------
# Run this new Scenario

resultSummary = run(myProject, scenario="No Harvest", jobs=6, summary=T)   # Uses multiprocessing
resultSummary

backup(myLibrary)  # Backup of your library

# Return to SyncroSim for Windows and Refresh to see that Results have been created
# *** Remember to add your scenario to the Active Results in order to then create a Chart
# Use the SyncroSim for Windows to view the review the model inputs and outputs
